var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const path = require('path');
const React = require('react');
const { parse } = require('url');
const { renderToString } = require('react-dom/server');

const browserConfig = {
  features: {
    FetchExternalResources: ['script'],
    ProcessExternalResources: ['script'],
    SkipExternalResources: false
  }
};

const Browser = function (jsdom, fs, buildDirectory, pathname, html) {
  this.fs = fs;
  this.jsdom = jsdom;
  this.html = html;
  this.pathname = pathname;
  this.host = parse(`http://${pathname}`).host;
  this.buildDirectory = buildDirectory;
};

Browser.prototype.resourceLoader = (() => {
  var _ref = _asyncToGenerator(function* (resource, callback) {
    const resourcePathname = path.join(this.buildDirectory, resource.url.pathname);

    if (resource.url.host !== this.host) {
      return callback();
    }

    try {
      const content = yield this.fs.readFileSync(resourcePathname);
      callback(null, content);
    } catch (error) {
      console.error(error);
      callback(error);
    }
  });

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
})();

Browser.prototype.created = function (resolve, reject, error, window) {
  if (error) {
    console.error(error);
  }
  this.jsdom.changeURL(window, 'http://' + this.pathname);
};

Browser.prototype.done = (() => {
  var _ref2 = _asyncToGenerator(function* (resolve, reject, error, window) {
    const app = window.app;

    if (!app) {
      return reject('No application found.');
    }

    let initialProps;

    if (typeof app.getInitialProps === 'function') {
      try {
        initialProps = yield app.getInitialProps();
      } catch (error) {
        return reject(error);
      }
    } else {
      initialProps = {};
    }

    const content = renderToString(React.createElement(app, initialProps));

    window.document.getElementById('root').innerHTML = content;

    if (initialProps) {
      const script = window.document.createElement('script');
      const stringifiedProps = JSON.stringify(initialProps);
      script.text = `window.__INITIAL_PROPS__ = ${stringifiedProps}`;
      const firstScript = window.document.body.getElementsByTagName('script')[0];
      window.document.body.insertBefore(script, firstScript);
    }

    resolve(window.document.documentElement.outerHTML);
  });

  return function (_x3, _x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  };
})();

Browser.prototype.execute = function () {
  return new Promise((resolve, reject) => {
    this.jsdom.env(_extends({
      html: this.html
    }, browserConfig, {
      resourceLoader: this.resourceLoader.bind(this),
      virtualConsole: this.jsdom.createVirtualConsole().sendTo(console),
      created: this.created.bind(this, resolve, reject),
      done: this.done.bind(this, resolve, reject)
    }));
  });
};

const BrowserFactory = function (jsdom, fs, buildDirectory) {
  return Browser.bind(null, jsdom, fs, buildDirectory);
};

module.exports = BrowserFactory;