function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const path = require('path');

const Snapshot = function (manifest, fs, browser, snapshotDirectory, applicationPath, validity) {
  this.fs = fs;
  this.browser = browser;
  this.validity = validity;
  this.manifest = manifest;
  this.applicationPath = applicationPath;
  this.snapshotDirectory = snapshotDirectory;
};

Snapshot.prototype.getPath = function (pathname) {
  const filename = path.basename(pathname) === '' ? `${pathname}index.snapshot` : `${pathname}.snapshot`;

  return path.join(this.snapshotDirectory, filename);
};

Snapshot.prototype.exists = function (pathname) {
  const snapshotPath = this.getPath(pathname);
  return this.manifest.exists(snapshotPath);
};

Snapshot.prototype.isValid = function (pathname) {
  if (this.validity === 0) {
    return true;
  }

  const snapshotPath = this.getPath(pathname);
  return this.manifest.isValid(snapshotPath);
};

Snapshot.prototype.getApplication = function () {
  try {
    return this.fs.readFileSync(this.applicationPath);
  } catch (error) {
    console.error(error);
    throw new Error(`
      Could not find a build index.html, please run 'yarn build'
      before running create-snapshot-server.
    `);
  }
};

Snapshot.prototype.create = (() => {
  var _ref = _asyncToGenerator(function* (pathname) {
    const html = this.getApplication();
    const browser = new this.browser(pathname, html);
    return yield browser.execute();
  });

  return function (_x) {
    return _ref.apply(this, arguments);
  };
})();

Snapshot.prototype.getPath = function (pathname) {
  const filename = path.basename(pathname) === '' ? `${pathname}index.snapshot` : `${pathname}.snapshot`;

  return path.join(this.snapshotDirectory, filename);
};

Snapshot.prototype.save = function (pathname, contents) {
  const snapshotPath = this.getPath(pathname);

  const dirname = path.dirname(snapshotPath);

  if (!this.fs.existsSync(dirname)) {
    this.fs.mkdirSync(dirname);
  }

  this.manifest.add(snapshotPath);

  return this.fs.writeFile(snapshotPath, contents).catch(error => console.error(error));
};

module.exports = Snapshot;